#-*- coding:utf-8 -*-

from functools import wraps
import json

from django.core.urlresolvers import reverse
from django.db import transaction
from django.core.paginator import Paginator
from django.http import HttpResponse
from django.shortcuts import get_object_or_404 as get_object
from django.shortcuts import render, redirect
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.utils.decorators import available_attrs
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView, ListView, DetailView
from django.views.generic.edit import FormView, CreateView, UpdateView

from app_settings import *
from .constants import MARKUP_EDITORS
from models import MessageBox, Message
from models import Category
from models import Forum
from models import Profile
from forms import ReplyForm, ThreadForm
from forms import ProfileForm, RateForm
from forms import MessageForm
from urlparse import urlparse, urljoin


def is_path(target):
    parsed = urlparse(target)
    return parsed.netloc == '' and parsed.scheme == ''


def is_internal_url(request, target):
    ref_url = urlparse(request.get_host())
    test_url = urlparse(urljoin(request.get_host(), target))
    return test_url.scheme in ('http', 'https') and \
           ref_url.netloc == test_url.netloc


def user_passes_test(test_func, redirect_field_name):
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):
            if test_func(request.user):
                return view_func(request, *args, **kwargs)

            return redirect(reverse('forum:register_profile') +
                            '?next=' + request.get_full_path())

        return _wrapped_view

    return decorator


def profile_required(function=None, redirect_field_name='next'):
    actual_decorator = user_passes_test(
        lambda u: hasattr(u, 'forum_profile'),
        redirect_field_name
    )

    if function:
        return actual_decorator(function)
    return actual_decorator


class ProtectedMixin(object):
    _profile = None

    @method_decorator(login_required)
    @method_decorator(profile_required)
    def dispatch(self, *args, **kwargs):
        return super(ProtectedMixin, self).dispatch(*args, **kwargs)

    @property
    def profile(self):
        return self.request.user.forum_profile


class IndexView(TemplateView):
    template_name = 'forum/index.html'

    def get(self, request):
        return self.render_to_response({
            'category_set': Category.objects.active()
        })


class PublicProfileView(TemplateView):
    template_name = 'forum/profile.html'

    def get(self, request, nickname):
        return self.render_to_response({
            'is_self': False,
            'profile': get_object(Profile, nickname=nickname)
        })


class ProfileView(ProtectedMixin, TemplateView):
    template_name = 'forum/profile.html'

    def get(self, request):
        return self.render_to_response({
            'is_self': True,
            'profile': request.user.forum_profile
        })


class EditProfileView(ProtectedMixin, UpdateView):
    model = Profile
    form_class = ProfileForm
    template_name = 'forum/edit_profile.html'

    def form_valid(self, form):
        messages.info(self.request, _('Profile updated'))
        return super(EditProfileView, self).form_valid(form)

    def get_success_url(self):
        return reverse(self.kwargs.get('next') or 'forum:profile')

    def get_object(self, queryset=None):
        return self.profile

    def get_context_data(self, **kwargs):
        context_data = super(EditProfileView, self).get_context_data(**kwargs)
        context_data['profile'] = self.profile
        return context_data


class SendMessageView(ProtectedMixin, CreateView):
    _recipient = None
    model = Message
    form_class = MessageForm
    template_name = 'forum/send_message.html'

    @property
    def recipient(self):
        nickname = self.kwargs['nickname']
        self._recipient = self._recipient \
            or get_object(Profile, nickname=nickname)
        return self._recipient

    def form_valid(self, form):
        form.save(self.profile, self.recipient)
        messages.success(self.request, _('Message sent.'))
        return redirect(self.get_success_url())

    def get_success_url(self):
        return reverse('forum:messages')

    def get_context_data(self, **kwargs):
        context = super(SendMessageView, self).get_context_data(**kwargs)
        context['profile'] = self.profile
        context['recipient'] = self.recipient
        return context


class UserMessageView(ProtectedMixin, DetailView):
    pk_url_kwarg = 'pk_message'
    template_name = 'forum/message.html'
    context_object_name = 'pvt'

    def get_queryset(self):
        profile = self.request.user.forum_profile
        return profile.messages.active()

    def get_context_data(self, **kwargs):
        context = super(UserMessageView, self).get_context_data(**kwargs)
        context['profile'] = self.profile
        return context

    def get(self, request, *args, **kwargs):
        self.get_object().read(commit=True)
        return super(UserMessageView, self).get(request, *args, **kwargs)


class UserMessagesView(ProtectedMixin, ListView):
    template_name = 'forum/messages.html'
    paginate_by = PAGINATE_MESSAGES_BY

    def get_queryset(self):
        profile = self.request.user.forum_profile
        return profile.messages.active()

    def get_context_data(self, **kwargs):
        context = super(UserMessagesView, self).get_context_data(**kwargs)
        context['profile'] = self.request.user.forum_profile
        context['new_messages'] = self.get_queryset().filter(was_read=False)
        return context


class CategoryView(TemplateView):
    template_name = 'forum/index.html'

    def get(self, request, *args, **kwargs):
        category_set = Category.objects.active().filter(pk=self.kwargs['pk_category'])
        return self.render_to_response({
            'category_set': category_set
        })


class ForumView(TemplateView):
    template_name = 'forum/forum.html'

    def get_context_data(self, **kwargs):
        pk_forum = self.kwargs['pk_forum']
        forum = get_object(Forum.objects.active(), pk=pk_forum)
        category = forum.category
        threads = forum.threads.active()
        page_number = self.kwargs.get('page', 1)
        object_list = Paginator(threads, PAGINATE_THREADS_BY).page(page_number)
        context = super(ForumView, self).get_context_data(**kwargs)
        context['category'] = category
        context['forum'] = forum
        context['object_list'] = object_list
        return context


class ThreadView(TemplateView):
    template_name = 'forum/thread.html'


def thread_view(request, pk_forum, pk_thread):
    forum = get_object(Forum.objects.active(), pk=pk_forum)
    category = forum.category
    threads = forum.threads.active()
    thread = get_object(threads, pk=pk_thread)
    replies = thread.replies.active()
    page_number = request.GET.get('page', 1)
    object_list = Paginator(replies, PAGINATE_REPLIES_BY).page(page_number)
    form = None

    # FIXME - form reference problem
    if ALLOW_INLINE_REPLY:
        form = ReplyForm(request.POST or None)

        if form.is_bound and form.is_valid():
            new_reply = form.save()
            return redirect(new_reply)

    thread.update_views(request)
    return render(request, 'forum/thread.html', {
        'category': category,
        'form': form,
        'forum': forum,
        'thread': thread,
        'object_list': object_list
    })


@login_required
@profile_required
@transaction.atomic
def new_thread_view(request, pk_forum, pk_thread=None):
    forum = get_object(Forum.objects.active(), pk=pk_forum)
    category = forum.category

    thread_form = ThreadForm(
        request.POST or None,
        request.method == 'POST' and request.FILES or None)
    reply_form = ReplyForm(
        request.POST or None,
        request.method == 'POST' and request.FILES or None)
    profile = request.user.forum_profile

    if thread_form.is_bound and reply_form.is_bound \
            and thread_form.is_valid() and reply_form.is_valid():
        new_thread = thread_form.save(profile, forum)
        reply_form.save(profile, forum, new_thread)
        return redirect(new_thread)
    return render(request, 'forum/new_thread.html', {
        'category': category,
        'forum': forum,
        'thread_form': thread_form,
        'reply_form': reply_form
    })


@login_required
@profile_required
@transaction.atomic
def reply_view(request, pk_forum, pk_thread):
    forum = get_object(Forum.objects.active(), pk=pk_forum)
    category = forum.category
    threads = forum.threads.active()
    thread = get_object(threads, pk=pk_thread)
    form = ReplyForm(request.POST or None)
    editor_name = MARKUP_EDITORS[MARKUP]
    editor_style = 'js/forum/markitup/sets/%s/style.css' % editor_name
    editor_set = 'js/forum/markitup/sets/%s/set.js' % editor_name

    if form.is_bound and form.is_valid():
        new_reply = form.save(request.user.forum_profile, forum, thread)
        return redirect(new_reply)

    return render(request, 'forum/reply.html', {
        'category': category,
        'forum': forum,
        'thread': thread,
        'form': form,
        'editor_name': editor_name,
        'editor_style': editor_style,
        'editor_set': editor_set,
    })


@login_required
@profile_required
def rate_reply_view(request, pk_forum, pk_thread, pk_reply):
    forum = get_object(Forum.objects.active(), pk=pk_forum)
    threads = forum.threads.active()
    thread = get_object(threads, pk=pk_thread)
    reply = get_object(thread.replies.active(), pk=pk_reply)

    form = RateForm(request.POST or None)
    if form.is_bound and form.is_valid():
        rating = form.save(request.profile, reply)
        return HttpResponse(
            json.dumps({'status': 'ok', 'message': 'success'}),
            mimetype='application/json')

    return HttpResponse(
        json.dumps({'status': 'error', 'message': 'invalid form data'}),
        mimetype='application/json')


@login_required
def register_profile_view(request):
    next = request.REQUEST.get('next', None)

    if request.method == 'POST':
        profile, created = Profile.objects.get_or_create(user=request.user)
        box = MessageBox.objects.get_or_create(profile=profile)
        messages.info(request, _("You now have a forum profile! Please, enjoy ; )"))

        if next is not None and is_path(next):
            return redirect(next)

        return redirect('forum:index')

    return render(request, 'forum/register_profile.html',
                  {'next': next})


# -*- coding:utf-8 -*-

from app_settings import *
from validators import max_upload_size, virus_check
from models import Reply, Thread, Profile, Message, Rating

from django import forms
from django.utils.translation import ugettext_lazy as _

THREAD_META_FIELDS = ['title']

if THREAD_ALLOW_IMAGE:
    THREAD_META_FIELDS.append('image')
if THREAD_ALLOW_ATTACHMENT:
    THREAD_META_FIELDS.append('attachment')


class MessageForm(forms.ModelForm):

    def __init__(self, *args, **kw):
        super(MessageForm, self).__init__(*args, **kw)
        self.fields['message'].required = True

    def save(self, author, recipient, commit=True):
        message = super(MessageForm, self).save(commit=False)
        message.author = author
        message.to = recipient

        if commit:
            message.save()

        return message

    class Meta:
        model = Message
        fields = ['title', 'message']


class ReplyForm(forms.ModelForm):
    """
    Form used to create a reply
    """

    class Meta:
        model = Reply
        fields = ['message']

    def __init__(self, *args, **kw):
        super(ReplyForm, self).__init__(*args, **kw)
        self.fields['message'].required = True
        self.fields['message'].widget.attrs['class'] = 'markitup'

    def save(self, author, forum, thread, commit=True):
        reply = super(ReplyForm, self).save(commit=False)
        reply.author = author
        reply.forum = forum
        reply.thread = thread

        if commit:
            reply.save()

        return reply


class ThreadForm(forms.ModelForm):
    if THREAD_ALLOW_IMAGE:
        __field = Thread._meta.get_field('image')
        image = forms.ImageField(
            label = unicode(__field.verbose_name),
            help_text = unicode(__field.help_text),
            validators = [max_upload_size(MAX_THREAD_IMAGE_SIZE)],
            required = not __field.blank
        )

    if THREAD_ALLOW_ATTACHMENT:
        __field = Thread._meta.get_field('attachment')
        attachment = forms.FileField(
            label = unicode(__field.verbose_name),
            help_text = unicode(__field.help_text),
            validators = [max_upload_size(MAX_THREAD_ATTACH_SIZE), virus_check],
            required = not __field.blank
        )

    class Meta:
        model = Thread
        fields = THREAD_META_FIELDS

    def save(self, author, forum, commit=True):
        thread = super(ThreadForm, self).save(commit=False)
        thread.author = author
        thread.forum = forum

        if commit:
            thread.save()

        return thread


class ProfileAdminForm(forms.ModelForm):
    class Meta:
        model = Profile
        exclude = ['flag']


class ProfileForm(forms.ModelForm):
    message = forms.CharField(
        label = _('Signature'),
        widget = forms.Textarea,
        required = not Profile._meta.get_field('message').blank)
    avatar = forms.ImageField(
        label=Profile._meta.get_field('avatar').verbose_name.title(),
        validators=[max_upload_size(AVATAR_MAX_SIZE)],
        required = not Profile._meta.get_field('avatar').blank
    )

    class Meta:
        model = Profile
        fields = ['gender', 'nickname', 'avatar', 'message']
        exclude = ['user', 'flag', 'reputation', 'threads_counter', 'replies_counter']


class RateForm(forms.ModelForm):
    def save(self, author, reply, **kw):
        self.author = author
        self.reply = reply
        return super(RateForm, self).save(**kw)

    class Meta:
        model = Rating


# -*- coding:utf-8 -*-
__all__ = ('max_upload_size', 'content_type_check', 'virus_check')

import logging

from django.core.exceptions import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

log = logging.Logger(__name__)

try:
    import pyclamd

    scan_file = lambda x: [0]

    clam_socket = pyclamd.ClamdUnixSocket()
    clam_socket.ping()
    scan_file = clam_socket.scan_file
except ImportError, e:
    log.warn(ugettext('Please, install clamav and pyclamd if you want virus checking for user uploaded files.'))
except pyclamd.ConnectionError:
        pass

try:
    import magic

    m = magic.open(magic.MAGIC_MIME)
    m.load()
    guess_type = m.file
except ImportError:
    import mimetypes

    guess_type = lambda x: mimetypes.guess_type(x)[0]
    log.warn(ugettext('Please, install python-magic package for better mimetype discovery support.'))


def max_upload_size(max_size):
    """
    Validator that ensures the maximum size of a file.
    """

    def _max_upload_size(file, _max_size):
        if file.size > _max_size:
            error_msg = 'Please keep filesize under %(max_filesize)s. Current filesize %(cur_filesize)s.'
            raise ValidationError(ugettext(error_msg) % {
                'max_filesize': filesizeformat(_max_size),
                'cur_filesize': filesizeformat(file.size)
            })

    return lambda value: _max_upload_size(value, max_size)


def virus_check(file):
    """
    Validator that checks if a uploaded file is infected. It uses pyclamav to do so.
    It requires that uploadhandler MemoryFileUploadHandler is not installed.
    """
    if hasattr(file, 'temporary_file_path'):
        scan_result = scan_file(file.temporary_file_path)

        # TODO: logging needed
        if scan_result is not None:
            log.info('Attachment %s is unsafe.' % file.name)
            raise ValidationError(ugettext('File is considered unsafe. Upload aborted.'))
        else:
            log.info('Attachment %s is safe.' % file.name)
    else:
        log.warn(ugettext('Please, make sure clamav and pyclamd are installed. Virus check not performed.'))


def content_type_check(content_types):
    """
    Makes sure the uploaded file is of a certain type. If python-magic is installed,
    mimetype guessing is done using it. Otherwise, mimetypes module is used.
    It requires that uploadhandler MemoryFileUploadHandler is not installed.
    """

    def _content_type_check(file, _content_types):
        if hasattr(file, 'temporary_file_path'):
            content_type = guess_type(file.temporary_file_path)
            if content_type not in _content_types:
                raise ValidationError(_('Filetype not supported.'))
        else:
            log.warn(ugettext('Please, make sure clamav and pyclamav are installed. Virus check not performed.'))

    return lambda value: _content_type_check(value, content_types)



# -*- coding:utf-8 -*-

from django.contrib import admin

from .forms import ProfileAdminForm
from .models import Category, Forum, Thread, Reply, Profile


class ProfileAdmin(admin.ModelAdmin):
    form = ProfileAdminForm


class ForumInlineModel(admin.StackedInline):
    model = Forum
    exclude = ('parent',)
    extra = 3
    max_num = 6


class CategoryAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ('name',)}
    inlines = [ForumInlineModel]

admin.site.register(Category, CategoryAdmin)
admin.site.register(Forum)
admin.site.register(Thread)
admin.site.register(Reply)
admin.site.register(Profile, ProfileAdmin)


# -*- coding:utf-8 -*-

from django.conf.urls import include, patterns, url
from .views import IndexView
from .views import UserMessageView, UserMessagesView, SendMessageView
from .views import PublicProfileView, ProfileView, EditProfileView
from .views import CategoryView, ForumView

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),

    # your own profile
    url(r'^profile/$', ProfileView.as_view(), name='profile'),

    url(r'^profile/register/',
        'forum.views.register_profile_view', name='register_profile'),

    # edit your own profile
    url(r'^profile/edit/$', EditProfileView.as_view(), name='edit_profile'),

    # someones profile
    url(r'^profile/(?P<nickname>[\w\-\+\.@]+)/$',
        PublicProfileView.as_view(), name='profile'),

    # send someone a message
    url(r'^profile/(?P<nickname>[\w\-\+\.@]+)/send_message/$',
        SendMessageView.as_view(), name='send_message'),

    # see your own messages
    url(r'^messages/$', UserMessagesView.as_view(), name='messages'),

    # read your own message
    url(r'^message/(?P<pk_message>\d+)/$', UserMessageView.as_view(), name='messages'),

    # see forum under this category
    url(r'^category/(?P<pk_category>[\w\-\+\.@]+)/$',
        CategoryView.as_view(), name='category'),

    # see forum
    url(r'^(?P<pk_forum>\d+)/$', ForumView.as_view(), name='expose'),

    # post a new thread
    url(r'^(?P<pk_forum>\d+)/threads/new/$',
        'forum.views.new_thread_view', name='new_thread'),

    # read a thread
    url(r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/$',
        'forum.views.thread_view', name='thread'),
#
#    # edit a thread
#    (r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/edit/$',
#        'edit_thread_view', {}, 'edit_thread'),

    # reply to a thread
    url(r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/reply/$',
        'forum.views.reply_view', name='reply'),

#    # rate a thread
#    (r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/reply/(?P<pk_reply>\d+)/rate/$',
#        'rate_reply_view', {}, 'rate'),
)



# -*- coding:utf-8 -*-
from django.utils.translation import ugettext

KB = 1024  # bytes
MB = 1024 * KB  # kb

GENDER_UNDEFINED = 'I'
GENDER_MALE = 'M'
GENDER_FEMALE = 'F'
GENDER_CONFUSED = 'W'
GENDER_CHOICES = (
    (GENDER_UNDEFINED, '???'),
    (GENDER_MALE, ugettext('Male')),
    (GENDER_FEMALE, ugettext('Female')),
    (GENDER_CONFUSED, ugettext("Won't tell"))
)

BBCODE, RST, MARKDOWN, TEXTILE, PLAIN_HTML, PLAIN_TEXT = 0, 1, 2, 3, 4, 5

TEXT_FORMAT_CHOICES = (
    (BBCODE, 'BBcode'),
    (RST, 'reStructuredText'),
    (MARKDOWN, 'Markdown'),
    (TEXTILE, 'Textile'),
    (PLAIN_HTML, 'Plain HTML'),
    (PLAIN_TEXT, 'Plain text'),
)

# plain text, bbcode, markdown, textile, restructuredtext, plain html
MARKUP_CHOICES = (
    (PLAIN_TEXT, 'Plain text'),
    (BBCODE, 'BBCode'),
    (MARKDOWN, 'Markdown'),
    (TEXTILE, 'Textile'),
    (RST, 'reStructuredText'),
    (PLAIN_HTML, 'Plain HTML'),
)

MARKUP_EDITORS = {
    PLAIN_TEXT: 'default',
    BBCODE: 'bbcode',
    MARKDOWN: 'markdown',
    TEXTILE: 'textile',
    RST: 'rest',
    PLAIN_HTML: 'html',
}

TRANSLATE_MARKUP = {}

for code, txt in MARKUP_CHOICES:
    TRANSLATE_MARKUP[txt.lower()] = code

# reply ratings
LIKES, DISLIKES = 1, -1
RATING_CHOICES = (
    (DISLIKES, 'Dislikes'),
    (LIKES, 'Liked'),
)

# thread status
# thread is open -> can view, *edit and reply
THREAD_STATUS_OPEN = 2**1
# thread is open -> can view, can not edit nor reply
THREAD_STATUS_CLOSED = 2**2
THREAD_STATUS_FLAGGED = 2**3
THREAD_STATUS_CH = (
    (THREAD_STATUS_OPEN, ugettext('Open')),
    (THREAD_STATUS_CLOSED, ugettext('Closed')),
    (THREAD_STATUS_FLAGGED, ugettext('Flagged'))
    )

PERMISSION_OPEN = 2**1
PERMISSION_LOGGED = 2**2
PERMISSION_PRIVATE = 2**3

PERMISSIONS = (
    # everybody can enter
    (PERMISSION_OPEN, ugettext('Open access')),
    # only logged users can enter
    (PERMISSION_LOGGED, ugettext('Permited to logged users only')),
    )

IMPROPER_CONTENT = 2**1
SPAM = 2**2
TROLL = 2**3
OTHER = 2**31

REASON_CH = (
    (IMPROPER_CONTENT, ugettext('Improper content')),
    (SPAM, ugettext('Spam')),
    (TROLL, ugettext('Trolling')),
    (OTHER, ugettext('Other reason')),
    )

FLAG_NEW = 2**1
FLAG_READEN = 2**2

PVT_MESSAGE_CH = (
    (FLAG_NEW, ugettext('New message')),
    (FLAG_READEN, ugettext('Readen')),
    )

NICK_NOVICE = 0
NICK_MEDIUM = 1
NICK_HIGH = 2
NICK_MASTER = 3
NICK_CH = (
    (NICK_NOVICE, ugettext('Slave')),
    (NICK_MEDIUM, ugettext('Padawan')),
    (NICK_HIGH, ugettext('Jedi')),
    (NICK_MASTER, ugettext('Lord Siph')),
)


# -*- coding:utf-8 -*-

"""
Only bbcode is supported so far.
"""

from .constants import BBCODE, MARKDOWN, TEXTILE
from .constants import RST, PLAIN_HTML, PLAIN_TEXT

# FIXME: setup logging

try:
    import textile
except ImportError:
    textile = None
    print 'textile module not found. Textile support is off.'

try:
    import markdown
except ImportError:
    markdown = None
    print 'markdown module not found. Markdown support is off.'

try:
    import bbcode
except ImportError:
    bbcode = None
    print 'bbcode module not found. BBCode support is off.'

try:
    from docutils.core import publish_parts
except ImportError:
    publish_parts = None
    print 'docutils module not found. RST support is off.'

try:
    import html5lib
    from html5lib import sanitizer

    html_sanitizer = html5lib.HTMLParser(tokenizer=sanitizer.HTMLSanitizer)
except ImportError:
    html_sanitizer = None
    print 'html5lib not found. Plain html support is off.'


def render_markup(markup, text):
    if text is None:
        return None

    if markup == BBCODE:
        if bbcode is not None:
            return bbcode.render_html(text)
    elif markup == MARKDOWN:
        if markdown is not None:
            return markdown.markdown(text)
    elif markup == TEXTILE:
        if textile is not None:
            return textile.textile(text)
    elif markup == RST:
        if publish_parts is not None:
            return publish_parts(text, writer_name='html')['fragment']
    elif markup == PLAIN_HTML:
        if html_sanitizer is not None:
            return html_sanitizer.parse(text)
    return text


# -*- coding:utf-8 -*-

import os
import datetime

from django.conf import settings
from django.db import models
from django.db.models import Sum, Count
from django.db.models.signals import pre_save, post_save, post_delete
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.template.defaultfilters import filesizeformat
from django.core.urlresolvers import reverse

from .constants import *
from .markup import render_markup
from .exceptions import *
from app_settings import *


#from .model_fields import RestrictedFileField, RestrictedImageField


def forum_profile_for(user):
    instance, created = Profile.objects.get_or_create(user=user)
    return instance


# --- Custom managers
class ActiveManager(models.Manager):
    def active(self):
        return super(ActiveManager, self).filter(is_active=True)


# --- Abstract classes
class DatedModel(models.Model):
    creation_time = models.DateTimeField(auto_now_add=True)
    modification_time = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class ActiveModel(models.Model):
    is_active = models.BooleanField(verbose_name='Is active?', default=True)
    objects = ActiveManager()

    class Meta:
        abstract = True


class MessageModel(models.Model):
    """
    Used for models that should have a formatted message attached.
    """
    message = models.TextField(blank=True, null=True)
    message_format = models.IntegerField(
        choices=MARKUP_CHOICES,
        default=MARKUP)
    html = models.TextField(editable=False, blank=True, null=True)

    class Meta:
        abstract = True


# --- End abstract classes


class Profile(DatedModel, ActiveModel, MessageModel):
    """
    Forum user profile
    """
    user = models.OneToOneField(User, related_name="forum_profile", primary_key=True)
    nickname = models.SlugField(max_length=50, unique=True, null=False)
    gender = models.CharField(max_length=1, choices=GENDER_CHOICES, default=GENDER_UNDEFINED)
    avatar = models.ImageField(
        verbose_name=_('Avatar image'),
        help_text=_('Up to %s') % filesizeformat(AVATAR_MAX_SIZE),
        upload_to='uploads/forum/profiles/avatar/', blank=True, null=True)
    flag = models.IntegerField(default=0, help_text=_('System wise attribute'))

    reputation = models.PositiveIntegerField(default=0)
    threads_counter = models.PositiveIntegerField(default=0)
    replies_counter = models.PositiveIntegerField(default=0)

    class Meta:
        db_table = 'forum_profile'
        verbose_name = 'Profile'
        verbose_name_plural = 'Profiles'
        ordering = ['-creation_time']

    def __unicode__(self):
        return unicode(self.nickname or self.user)

    @models.permalink
    def get_absolute_url(self):
        return 'forum:profile', [self.nickname]

    def set_flag(self, flag):
        self.flag = self.flag | flag

    def get_avatar_url(self):
        if self.avatar:
            return self.avatar.url
        return settings.STATIC_URL + "img/forum/avatar/dull_face.png"

    def update_threads_counter(self):
        self.threads_counter = self.threads.count()

    def update_replies_counter(self):
        self.replies_counter = self.replies.count()

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):

        # TODO: request nickname on registering
        if not self.nickname:
            self.nickname = self.user.username

        return super(Profile, self).save(force_insert, force_update, using, update_fields)


class Category(DatedModel, ActiveModel):
    """
    Used to sort the forum
    """
    name = models.CharField(max_length=50)
    slug = models.SlugField(max_length=50, unique=True, primary_key=True)

    class Meta:
        db_table = 'forum_category'
        verbose_name = 'Category'
        verbose_name_plural = 'Categories'
        ordering = ['slug']

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return 'forum:category', [self.slug]


class Forum(DatedModel, ActiveModel):
    category = models.ForeignKey(
        Category, help_text='Set to show forum in the index page',
        related_name='forum_set', blank=True, null=True)
    parent = models.ForeignKey('self', related_name='children', blank=True, null=True, default=None)
    name = models.CharField(verbose_name=_("Forum name"), max_length=100)
    description = models.CharField(
        verbose_name=_("Description"),
        max_length=255, blank=True, null=True)
    moderation = models.ManyToManyField(
        'Profile', related_name="moderation",
        blank=True, null=True)
    last_thread = models.OneToOneField(
        'Thread', related_name="as_last_thread_for",
        blank=True, null=True, editable=False)

    threads_counter = models.PositiveIntegerField(_("Threads counter"), default=0, editable=False)
    replies_counter = models.PositiveIntegerField(_("Replies counter"), default=0, editable=False)

    class Meta:
        db_table = 'forum'
        verbose_name = 'Forum'
        verbose_name_plural = 'Forums'
        ordering = ['-creation_time']

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return 'forum:expose', [self.id]

    def parent_tree(self):
        parent_list = []
        parent = self.parent

        while parent is not None:
            parent_list.append(parent)
            parent = parent.parent

        return parent_list

    def latest_thread(self):
        try:
            return self.threads.active().latest('creation_time')
        except Thread.DoesNotExist:
            return None

    def check_permission(self, request):
        if self.access_filter == PERMISSION_LOGGED and not request.user.is_authenticated():
            raise LoginRequiredException()

    def update_threads_counter(self):
        self.threads_counter = self.threads.count()

    def update_replies_counter(self):
        agg = self.threads.aggregate(Sum('replies_counter'))
        self.replies_counter = agg['replies_counter__sum']


class Thread(DatedModel, ActiveModel):
    _order_by = ('title', 'author', 'replies_counter', 'view_counter')

    author = models.ForeignKey(Profile, related_name='threads')
    forum = models.ForeignKey(Forum, related_name='threads')
    status = models.IntegerField(
        choices=THREAD_STATUS_CH,
        default=THREAD_STATUS_OPEN)
    title = models.CharField(verbose_name=_('Title'), max_length=255)
    image = models.ImageField(
        verbose_name=_('Head image'), help_text='Shown at the threads top',
        #validators=[max_upload_size(MAX_THREAD_IMAGE_SIZE)],
        upload_to='uploads/forum/threads/image/',
        blank=True, null=True)
    attachment = models.FileField(
        verbose_name=_('Attachment'), help_text='Associate a file with this threads',
        #validators=[max_upload_size(MAX_THREAD_ATTACH_SIZE), virus_check],
        upload_to='uploads/forum/threads/attachment/',
        blank=True, null=True)

    views_counter = models.PositiveIntegerField(
        _("Views counter"), default=0, editable=False)
    replies_counter = models.PositiveIntegerField(
        _("Replies counter"), default=0, editable=False)
    authors_counter = models.PositiveIntegerField(
        _("Authors counter"), default=0, editable=False)

    class Meta:
        db_table = 'forum_thread'
        verbose_name = ugettext('Thread')
        verbose_name_plural = ugettext('Threads')
        ordering = ['-creation_time']

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return 'forum:thread', (self.forum_id, self.id), {}

    def get_attachment_basename(self):
        return os.path.basename(self.attachment.name)

    def update_views(self, request):
        """
        Updates the threads visit counter
        """
        ip_address = request.META['REMOTE_ADDR']
        return Views.objects.get_or_create(
            thread=self, ip_address=ip_address,
            date=datetime.date.today())

    def update_views_counter(self):
        self.views_counter = self.views.count()

    def update_replies_counter(self):
        self.replies_counter = self.replies.active().count()

    def update_authors_counter(self):
        agg = self.replies.active().aggregate(Count('author'))
        self.authors_counter = agg['author__count']


class Reply(DatedModel, ActiveModel, MessageModel):
    author = models.ForeignKey(Profile, related_name='replies')
    forum = models.ForeignKey(Forum, related_name='replies')
    thread = models.ForeignKey(Thread, related_name='replies')
    avg_rating = models.PositiveIntegerField(_('Average rating'), default=0)

    class Meta:
        db_table = 'forum_reply'
        verbose_name = ugettext('Reply')
        verbose_name_plural = ugettext('Replies')
        ordering = ['creation_time']

    def get_absolute_url(self):
        replies = self.thread.replies.active()
        page = int(list(replies).index(self) / PAGINATE_THREADS_BY) + 1
        return "%s?page=%d" % (reverse('forum:thread', args=[self.forum.id, self.thread.id]), page)

    def update_reputation(self):
        agg = self.rating.aggregate(Sum('value'))
        self.reputation = agg['value__sum']


class Views(models.Model):
    date = models.DateField(auto_now_add=True)
    thread = models.ForeignKey(Thread, related_name='views')
    ip_address = models.IPAddressField()

    class Meta:
        db_table = 'forum_views'
        verbose_name = ugettext('View')
        verbose_name_plural = ugettext('Views')
        ordering = ('-date', '-thread')
        unique_together = ('thread', 'ip_address', 'date')

    def __unicode__(self):
        return "%s" % self.ip_address


class Rating(DatedModel):
    """
    Stores a pontuation given from an user for a reply. This is used
    to calculate the total reputation of the user
    """
    reply = models.ForeignKey(Reply, related_name='rating')
    author = models.ForeignKey(Profile, related_name='ratings')
    value = models.IntegerField(choices=RATING_CHOICES)

    class Meta:
        db_table = 'forum_rating'
        verbose_name = 'Rating'
        verbose_name_plural = 'Ratings'

    def __unicode__(self):
        return self.get_value_display()


class AbuseReport(DatedModel):
    """
    An logged user can report a reply for abuse.
    """
    reply = models.ForeignKey(Reply, related_name='reports')
    author = models.ForeignKey('Profile', related_name='abuse_reports')
    reason = models.IntegerField(choices=REASON_CH)
    observation = models.TextField(_('Any observations?'), blank=True, help_text=_('Use plain text'))

    class Meta:
        db_table = 'forum_abuse_report'
        verbose_name = ugettext('Abuse report')
        verbose_name_plural = ugettext('Abuse reports')

    def __unicode__(self):
        return len(self.observation) > 30 and (self.observation[:30] + '...') or self.get_reason_display()


class Subscription(DatedModel, ActiveModel):
    """
    Represents a subscription of an user to a threads. That means a user
    is interested in the subscribed threads and wish to follow updates on it.
    """
    profile = models.ForeignKey('Profile', related_name="forum_subscribes")
    thread = models.ForeignKey(Thread, related_name="subscriptions")

    class Meta:
        db_table = 'forum_subscription'
        verbose_name = ugettext('Subscription')
        verbose_name_plural = ugettext('subscriptions')
        unique_together = ('profile', 'thread')

    def __unicode__(self):
        return _("Subscription to threads %s") % self.thread


class MessageBox(DatedModel):
    profile = models.OneToOneField(Profile, related_name='message_box')
    box_size = models.PositiveIntegerField(default=20)

    class Meta:
        db_table = 'forum_message_box'
        verbose_name = _('Message box')
        verbose_name_plural = _('Message boxes')


class Message(DatedModel, ActiveModel, MessageModel):
    """
    Represents a private message sent from an user to another.
    """
    was_read = models.BooleanField(verbose_name=_('Was read?'), default=False)
    was_read_at = models.DateTimeField(editable=False, blank=True, null=True)
    author = models.ForeignKey(Profile, verbose_name=_('Author'), related_name='sent_messages')
    to = models.ForeignKey(Profile, verbose_name=_('Recipient'), related_name='messages')
    title = models.CharField(max_length=100)

    class Meta:
        db_table = 'forum_message'
        verbose_name = ugettext('Message')
        verbose_name_plural = ugettext('Messages')
        ordering = ['creation_time']

    @models.permalink
    def get_absolute_url(self):
        return 'forum:messages', [self.id]

    def __unicode__(self):
        return self.title

    def read(self, commit=False):
        self.was_read = True
        self.was_read_at = datetime.datetime.now()

        if commit:
            self.save()


def on_new_profile(sender, instance, **kw):
    if kw.get('created'):
        MessageBox.objects.create(profile=instance)


post_save.connect(on_new_profile, sender=Profile,
                  dispatch_uid='forum_post_save_on_new_profile')


def on_new_views(sender, instance, **kw):
    """
    Updates the view counter for a threads
    """
    if kw.get('created', True):
        instance.thread.update_views_counter()
        instance.thread.save()


post_save.connect(on_new_views, sender=Views,
                  dispatch_uid='forum_post_save_on_new_views')


def on_new_thread(sender, instance, **kw):
    """
    Updates the threads_counter for the forum and threads author
    """
    # works for create and delete
    if kw.get('created', True) is True:
        instance.forum.update_threads_counter()
        instance.forum.save()

        instance.author.update_threads_counter()
        instance.author.save()


post_save.connect(on_new_thread, sender=Thread,
                  dispatch_uid='forum_thread_post_save_update_threads_counter')
post_delete.connect(on_new_thread, sender=Thread,
                    dispatch_uid='forum_thread_post_delete_update_threads_counter')


def on_new_rating(sender, instance, **kw):
    """
    Updates rating counter of the reply.
    """
    # works for create and delete
    if kw.get('created', True) is True:
        instance.reply.update_reputation()
        instance.reply.save()


post_save.connect(on_new_rating, sender=Rating,
                  dispatch_uid='forum_post_save_on_new_rating')
post_delete.connect(on_new_rating, sender=Rating,
                    dispatch_uid='forum_post_delete_on_new_rating')


def on_new_reply(sender, instance, **kw):
    """
    Actually this method name is misleading. The counters that are
    updated are, actually, the counters of the threads this reply is
    related to.
    """
    if kw.get('created', True) is True:
        instance.thread.update_authors_counter()
        instance.thread.update_replies_counter()
        instance.thread.save()

        # update reply counter for the author
        instance.author.update_replies_counter()
        instance.author.save()

        # sum all attributes replies_counter of all threads and updates forum
        instance.thread.forum.update_replies_counter()
        instance.thread.forum.save()


post_save.connect(on_new_reply, sender=Reply,
                  dispatch_uid='forum_post_save_on_new_reply')
post_delete.connect(on_new_reply, sender=Reply,
                    dispatch_uid='forum_post_delete_on_new_reply')


def update_html(sender, instance, *args, **kw):
    """
    pre_save event
    Renders the html for a reply and stores it in an attribute for
    performance
    """
    instance.html = render_markup(instance.message_format, instance.message)


pre_save.connect(update_html, sender=Profile,
                 dispatch_uid='forum_pre_save_update_profile_html')
pre_save.connect(update_html, sender=Reply,
                 dispatch_uid='forum_pre_save_update_reply_html')
pre_save.connect(update_html, sender=Message,
                 dispatch_uid='forum_pre_save_update_message_html')



# -*- coding:utf-8 -*-
import sys
import setuptools

setuptools.setup(
    name="django-forumbr",
    version="0.4",
    packages=[
        "forum",
        "forum.templatetags",
    ],

    install_requires=[
        "django>=1.5",
        "pillow",
        "south"
    ],
    tests_require=[
        'django-registration'
    ],

    author="Italo Maia",
    author_email="italo.maia@gmail.com",
    url="https://bitbucket.org/italomaia/django-forumbr",
    license="http://www.apache.org/licenses/LICENSE-2.0",
    description="Simple and complete django forum solution",
    keywords="django forum python",
    include_package_data=True,
)



# -*- coding:utf-8 -*-
"""
This file demonstrates two different styles of tests (one doctest and one
unittest). These will both pass when you run "manage.py test".

Replace these with more appropriate tests for your application.
"""

from random import randint
from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.contrib.auth.models import User


def new_user(username='john', email='email@email.com', password='doe'):
    return User.objects.create_user(username, email, password)


def new_profile(user, nickname=None, gender=None):
    from .models import Profile, GENDER_MALE

    return Profile.objects.create(
        user=user,
        nickname=nickname or user.username,
        gender=gender or GENDER_MALE)


class TestUrlBuild(TestCase):
    """
    Testa se todas as ulrs definidas em urls.py são geradas corretamente
    para os argumentos corretos.
    """
    slug = 'aAbBcCdDeE_-0123456789'
    object_id = 12345

    def test_index(self):
        reverse('forum:index')

    def test_view_profile(self):
        reverse('forum:profile')

    def test_view_public_profile(self):
        reverse('forum:profile', args=[self.slug])

    def test_edit_profile(self):
        reverse('forum:edit_profile')

    def test_view_messages(self):
        reverse('forum:messages')

    def test_view_message(self):
        reverse('forum:messages', args=[self.object_id])

    def test_view_send_message(self):
        reverse('forum:send_message', args=[self.slug])

    def test_view_forum(self):
        reverse('forum:expose', args=[self.object_id])

    def test_view_forum_thread(self):
        reverse('forum:thread', args=[self.object_id, 432])

    def test_forum_new_thread(self):
        reverse('forum:new_thread', args=[self.object_id])

    def test_forum_reply(self):
        reverse('forum:reply', args=[self.object_id, 432])


class TestIndexVew(TestCase):
    template = 'forum/index.html'

    def setUp(self):
        self.url = reverse('forum:index')

    def test_access_without_user_account(self):
        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, self.template)

    def test_access_without_profile(self):
        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, self.template)

    def test_access_with_profile(self):
        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, self.template)


class TestRegisterProfile(TestCase):
    pass  # TODO implement me!


class TestProfileView(TestCase):
    template = 'forum/profile.html'

    def setUp(self):
        self.url = reverse('forum:profile')

    def test_access_without_user_account(self):
        resp = self.client.get(self.url)
        self.assertRedirects(resp, settings.LOGIN_URL + '?next=' + self.url)

    def test_access_without_profile(self):
        User.objects.create_user('john', 'email@email.com', 'doe')
        self.client.login(username='john', password='doe')

        resp = self.client.get(self.url)
        self.assertRedirects(resp, reverse('forum:register_profile'))

    def test_access_with_profile(self):
        from forum.models import Profile

        user = User.objects.create_user('john', 'email@email.com', 'doe')
        Profile.objects.create(user=user, nickname='blah')
        self.client.login(username='john', password='doe')

        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, self.template)


class TestPublicProfileView(TestCase):
    template = 'forum/profile.html'

    def test_access_without_user_account(self):
        from .models import Profile

        user = User.objects.create_user('john', 'email@email.com', 'doe')
        Profile.objects.create(user=user, nickname='blah')

        url = reverse('forum:profile', args=['blah'])
        resp = self.client.get(url)
        self.assertTemplateUsed(resp, self.template)


class TestEditProfileView(TestCase):
    def setUp(self):
        self.url = reverse('forum:edit_profile')

    def test_access_without_user_account(self):
        resp = self.client.get(self.url)
        self.assertRedirects(resp, settings.LOGIN_URL + '?next=' + self.url)

    def test_access_without_profile(self):
        user = User.objects.create_user('john', 'email@email.com', 'doe')
        self.client.login(username='john', password='doe')

        resp = self.client.get(self.url)
        self.assertRedirects(resp, reverse('forum:register_profile'))

    def test_access_with_profile(self):
        user = User.objects.create_user('john', 'email@email.com', 'doe')
        profile = new_profile(user)

        self.client.login(username='john', password='doe')
        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, 'forum/edit_profile.html')


class TestSendMessageView(TestCase):
    def setUp(self):
        self.john = new_user()
        self.john_profile = new_profile(self.john)

        self.maria = new_user(username='maria')
        self.maria_profile = new_profile(self.maria)

    def test_access_send_msg_page_to_non_existing_nickname(self):
        self.client.login(username=self.john.username, password='doe')
        resp = self.client.get(reverse('forum:send_message', args=['blabla']))
        self.assertEqual(resp.status_code, 404)


class TestMessagesView(TestCase):
    def setUp(self):
        self.url = reverse('forum:messages')

    def test_access_without_user_account(self):
        resp = self.client.get(self.url)
        self.assertRedirects(resp, settings.LOGIN_URL + '?next=' + self.url)

    def test_access_without_profile(self):
        user = new_user()
        self.client.login(username='john', password='doe')

        resp = self.client.get(self.url)
        self.assertRedirects(resp, reverse('forum:register_profile'))

    def test_access_with_profile(self):
        user = new_user()
        profile = new_profile(user)

        self.client.login(username='john', password='doe')
        resp = self.client.get(self.url)
        self.assertTemplateUsed(resp, template_name='forum/messages.html')

    def test_messages_are_there(self):
        from .models import Message, GENDER_FEMALE

        first_profile = new_profile(
            new_user(username='john', password='doe'))

        second_profile = new_profile(
            new_user(username='maria', password='clara'),
            gender=GENDER_FEMALE)

        Message.objects.create(
            author=first_profile,
            to=second_profile,
            title='test message',
            message='this is a test')

        # authenticating our user
        self.client.login(username='maria', password='clara')

        resp = self.client.get(self.url)
        self.assertContains(resp, 'test message')


class TestCategoryView(TestCase):
    def test_access_to_category_view_without_forum(self):
        from .models import Category

        name, slug = 'some category', 'some-category'
        category = Category.objects.create(name=name, slug=slug)
        url = reverse('forum:category', args=[slug])
        resp = self.client.get(url)
        self.assertTemplateUsed(resp, 'forum/index.html')

    def test_access_to_category_view_with_forum(self):
        from .models import Category

        name, slug = 'some category', 'some-category'
        category = Category.objects.create(name=name, slug=slug)
        url = reverse('forum:category', args=[slug])
        resp = self.client.get(url)
        self.assertTemplateUsed(resp, 'forum/index.html')


class TestForumView(TestCase):
    def test_access_to_non_existing_forum(self):
        from .models import Forum

        resp = self.client.get(reverse('forum:expose', args=[100]))
        self.assertEqual(resp.status_code, 404)

    def test_access_to_forum_view_without_user_account(self):
        from .models import Forum

        forum = Forum.objects.create(name='some name')
        resp = self.client.get(reverse('forum:expose', args=[forum.id]))
        self.assertTemplateUsed(resp, 'forum/forum.html')


class TestThreadView(TestCase):
    def setUp(self):
        from .models import Forum, Thread, Reply

        self.user = new_user()
        self.profile = new_profile(self.user)
        self.forum = Forum.objects.create(name='bla bla')
        self.thread = Thread.objects.create(
            forum=self.forum,
            author=self.profile,
            title='some title')
        self.reply = Reply.objects.create(
            author=self.profile,
            forum=self.forum,
            thread=self.thread,
            message='some message')

    def test_access_to_thread_view_without_user_account(self):
        resp = self.client.get(reverse('forum:thread',
                                       args=[self.forum.id, self.thread.id]))
        self.assertTemplateUsed(resp, 'forum/thread.html')

    def test_thread_title_content(self):
        resp = self.client.get(reverse('forum:thread',
                                       args=[self.forum.id, self.thread.id]))
        self.assertContains(resp, 'some title')

    def test_reply_message_content(self):
        resp = self.client.get(reverse('forum:thread',
                                       args=[self.forum.id, self.thread.id]))
        self.assertContains(resp, 'some message')


class TestReplyView(TestCase):
    pass


# -*- coding:utf-8 -*-

from django.conf import settings
from .constants import KB, MB

MIN = 60

MARKUP_FORCE_SAFE_HTML = getattr(settings, 'FORUM_MARKUP_FORCE_SAFE_HTML', True)

# Markup language for threads and replies
# one of (plain text, bbcode, markdown, textile, restructuredtext, plain html)
MARKUP = getattr(settings, 'FORUM_MARKUP', 'plain text')

# Default: 120kb
MAX_THREAD_IMAGE_SIZE = getattr(settings, 'FORUM_MAX_THREAD_IMAGE_SIZE', 120 * KB)
# Default: 2mb
MAX_THREAD_ATTACH_SIZE = getattr(settings, 'FORUM_MAX_THREAD_ATTACH_SIZE', 2 * MB)

THREAD_ALLOW_IMAGE = getattr(settings, 'FORUM_THREAD_ALLOW_IMAGE', True)
THREAD_ALLOW_ATTACHMENT = getattr(settings, 'FORUM_THREAD_ALLOW_ATTACHMENT', True)

# max filesize for avatar image. Default: 40kb
AVATAR_MAX_SIZE = getattr(settings, 'FORUM_AVATAR_MAX_SIZE', 40 * KB)

# Can user edit reply?
EDIT_REPLY = getattr(settings, 'FORUM_EDIT_REPLY', False)
# How long, after posting can the user edit the reply? Set to None for unlimited time.
EDIT_REPLY_TIME_LIMIT = getattr(settings, 'FORUM_EDIT_REPLY_TIME_LIMIT', None)
# Add simple, inline reply functionality?
ALLOW_INLINE_REPLY = getattr(settings, 'FORUM_ALLOW_INLINE_REPLY', False)

# Requires clamav and pyclamd. Will complain otherwise.
CHECK_ATTACH_FOR_VIRUS = getattr(settings, 'FORUM_CHECK_ATTACHMENT_FOR_VIRUS', False)

# Pagination sizes
PAGINATE_THREADS_BY = getattr(settings, 'FORUM_PAGINATE_THREADS_BY', 16)
PAGINATE_REPLIES_BY = getattr(settings, 'FORUM_PAGINATE_REPLIES_BY', 12)
PAGINATE_MESSAGES_BY = getattr(settings, 'FORUM_PAGINATE_MESSAGES_BY', 20)

# Maximum number of messages an user can have in his message box
MESSAGE_BOX_SIZE = getattr(settings, 'FORUM_MESSAGE_BOX_SIZE', 40)

if CHECK_ATTACH_FOR_VIRUS:
    try:
        import pyclamd
    except ImportError:
        print('Option CHECK_ATTACH_FOR_VIRUS requires clamav and pyclamd installed.')
        exit(0)